@@ -25,6 +25,9 @@ extern G2C_FILE_INFO_T g2c_file[G2C_MAX_FILES + 1];
 /** Length of beginning of index record. */
 #define G2C_INDEX_FIXED_LEN 44
 
+/** Length of beginning of index record for large files. */
+#define G2C_INDEX_FIXED_LEN_2 48
+
 /** Length of date string in index record. */
 #define G2C_INDEX_DATE_STR_LEN 10
 
@@ -77,6 +80,8 @@ g2c_start_index_record(FILE *f, int rw_flag, int *reclen, int *msg, int *local,
         || !msglen || !version || !discipline || !fieldnum)
         return G2C_EINVAL;
 
+    LOG((4, "g2c_start_index_record rw_flag %d", rw_flag));
+
     /* When writing, set the fieldnum1 to be a 1-based index, just
      * like in Fortran. */
     if (rw_flag)
@@ -117,6 +122,88 @@ g2c_start_index_record(FILE *f, int rw_flag, int *reclen, int *msg, int *local,
     return G2C_NOERROR;
 }
 
+/**
+ * Read or write the start of a version 2 index record for large file.
+ *
+ * @param f FILE * to open index file.
+ * @param rw_flag True if function should write, false if it should read.
+ * @param reclen Pointer to reclen.
+ * @param msg Pointer to msg.
+ * @param local Pointer to local.
+ * @param gds Pointer to gds.
+ * @param pds Pointer to pds.
+ * @param drs Pointer to drs.
+ * @param bms Pointer to bms.
+ * @param data Pointer to data.
+ * @param msglen Pointer to msglen.
+ * @param version Pointer to version.
+ * @param discipline Pointer to discipline.
+ * @param fieldnum Pointer to fieldnum, 0- based. (It is 1-based in
+ * the index file.)
+ *
+ * @return
+ * - ::G2C_NOERROR No error.
+ * - ::G2C_EINVAL Invalid input.
+ * - ::G2C_EFILE File I/O error.
+ *
+ * @author Ed Hartnett 10/26/22
+ */
+int
+g2c_start_index_record_lf(FILE *f, int rw_flag, int *reclen, size_t *msg, int *local, int *gds,
+			  int *pds, int *drs, int *bms, int *data, size_t *msglen,
+			  unsigned char *version, unsigned char *discipline, short *fieldnum)
+{
+    /* size_t size_t_be; */
+    short fieldnum1; /* This is for the 1-based fieldnum in the index file. */
+    int ret;
+
+    LOG((4, "g2c_start_index_record_lf rw_flag %d", rw_flag));
+
+    /* All pointers must be provided. */
+    if (!f || !reclen || !msg || !local || !gds || !pds || !drs || !bms || !data
+        || !msglen || !version || !discipline || !fieldnum)
+        return G2C_EINVAL;
+
+    /* When writing, set the fieldnum1 to be a 1-based index, just
+     * like in Fortran. */
+    if (rw_flag)
+        fieldnum1 = *fieldnum + 1;
+
+    /* Read or write the values at the beginning of each index
+     * record. */
+    if ((ret = g2c_file_io_uint(f, rw_flag, (unsigned int *)reclen)))
+        return ret;
+    if ((ret = g2c_file_io_ulonglong(f, rw_flag, (unsigned long long *)msg)))
+        return ret;
+    if ((ret = g2c_file_io_uint(f, rw_flag, (unsigned int *)local)))
+        return ret;
+    if ((ret = g2c_file_io_uint(f, rw_flag, (unsigned int *)gds)))
+        return ret;
+    if ((ret = g2c_file_io_uint(f, rw_flag, (unsigned int *)pds)))
+        return ret;
+    if ((ret = g2c_file_io_uint(f, rw_flag, (unsigned int *)drs)))
+        return ret;
+    if ((ret = g2c_file_io_uint(f, rw_flag, (unsigned int *)bms)))
+        return ret;
+    if ((ret = g2c_file_io_uint(f, rw_flag, (unsigned int *)data)))
+        return ret;
+    if ((ret = g2c_file_io_ulonglong(f, rw_flag, (unsigned long long *)msglen)))
+        return ret;
+    if ((ret = g2c_file_io_ubyte(f, rw_flag, version)))
+        return ret;
+    if ((ret = g2c_file_io_ubyte(f, rw_flag, discipline)))
+        return ret;
+    if ((ret = g2c_file_io_short(f, rw_flag, &fieldnum1)))
+        return ret;
+
+    /* When reading, translate the 1-based fieldnum1 into the 0-based
+     * fieldnum that C programmers will expect and love. */
+    if (!rw_flag)
+        *fieldnum = fieldnum1 - 1;
+
+    return G2C_NOERROR;
+}
+
 /**
  * Read or write the start of a version 1 index record.
  *
@@ -354,6 +441,7 @@ g2c_write_index(int g2cid, int mode, const char *index_file)
     char my_path[G2C_INDEX_BASENAME_LEN + 1];
     G2C_MESSAGE_INFO_T *msg;
     int total_index_size = 0; /* Does not include size of header records. */
+    int index_version = 1; /* 1 for legacy, 2 if indexed file may be > 2 GB. */
     int reclen;
     int ret = G2C_NOERROR;
 
@@ -363,7 +451,7 @@ g2c_write_index(int g2cid, int mode, const char *index_file)
     if (!index_file)
         return G2C_EINVAL;
 
-    LOG((1, "g2c_write_index g2cid %d mode %d index_file %s", g2cid, mode,
+    LOG((2, "g2c_write_index g2cid %d mode %d index_file %s", g2cid, mode,
          index_file));
 
     /* If NOCLOBBER, check if file exists. */
@@ -377,6 +465,10 @@ g2c_write_index(int g2cid, int mode, const char *index_file)
         }
     }
 
+    /* If LARGE_INDEX_FILE, set index version. */
+    if (mode & G2C_LARGE_FILE_INDEX)
+	index_version = 2;
+    
     /* Create the index file. */
     if (!(f = fopen(index_file, "wb+")))
         return G2C_EFILE;
@@ -390,9 +482,10 @@ g2c_write_index(int g2cid, int mode, const char *index_file)
     if (!ret)
     {
         /* Create header 1. */
-        snprintf(h1, G2C_INDEX_HEADER_LEN + 1,
-                 "!GFHDR!  1   1   162 %4.4u-%2.2u-%2.2u %2.2u:%2.2u:%2.2u GB2IX1        hfe08           grb2index\n",
-                 (tm.tm_year + 1900), (tm.tm_mon + 1), tm.tm_mday, tm.tm_hour, tm.tm_min, tm.tm_sec);
+	snprintf(h1, G2C_INDEX_HEADER_LEN + 1,
+		 "!GFHDR!  1   1   162 %4.4u-%2.2u-%2.2u %2.2u:%2.2u:%2.2u %s        hfe08           grb2index\n",
+		 (tm.tm_year + 1900), (tm.tm_mon + 1), tm.tm_mday, tm.tm_hour, tm.tm_min, tm.tm_sec,
+		 "GB2IX1");
 
         /* Write header 1. */
         if ((items_written = fwrite(h1, G2C_INDEX_HEADER_LEN, 1, f)) != 1)
@@ -415,8 +508,8 @@ g2c_write_index(int g2cid, int mode, const char *index_file)
                     break;
                 
                 /* What will be the length of this index record? */
-                reclen = G2C_INDEX_FIXED_LEN + msg->sec1_len + sec3->sec_len + sec4->sec_len +
-                    sec5->sec_len + G2C_INDEX_BITMAP_BYTES;
+                reclen = (index_version == 1 ? G2C_INDEX_FIXED_LEN : G2C_INDEX_FIXED_LEN_2) +
+		    msg->sec1_len + sec3->sec_len + sec4->sec_len + sec5->sec_len + G2C_INDEX_BITMAP_BYTES;
                 total_index_size += reclen;
                 LOG((4, "fieldnum %d reclen %d total_index_size %d", fieldnum, reclen, total_index_size));
             } /* next product */
@@ -431,7 +524,7 @@ g2c_write_index(int g2cid, int mode, const char *index_file)
     if (!ret)
     {
         strncpy(my_path, basename(g2c_file[g2cid].path), G2C_INDEX_BASENAME_LEN);
-        sprintf(h2, "IX1FORM:       162    %6d    %6ld  %s    \n", total_index_size,
+        sprintf(h2, "IX%dFORM:       162    %6d    %6ld  %s    \n", index_version, total_index_size,
                 g2c_file[g2cid].num_messages, my_path);
         LOG((5, "header 2: %s", h2));
         
@@ -451,7 +544,6 @@ g2c_write_index(int g2cid, int mode, const char *index_file)
             for (fieldnum = 0; fieldnum < msg->num_fields; fieldnum++)
             {
                 G2C_SECTION_INFO_T *sec3, *sec4, *sec5, *sec6, *sec7;
-                int bytes_to_msg = (int)msg->bytes_to_msg;
                 int bs3, bs4, bs5, bs6, bs7; /* bytes to each section, as ints. */
                 unsigned char sec_num;
                 int ret;
@@ -465,15 +557,26 @@ g2c_write_index(int g2cid, int mode, const char *index_file)
                 bs7 = (int)sec7->bytes_to_sec;
 
                 /* What will be the length of this index record? */
-                reclen = G2C_INDEX_FIXED_LEN + msg->sec1_len + sec3->sec_len + sec4->sec_len +
-                    sec5->sec_len + G2C_INDEX_BITMAP_BYTES;
+                reclen = (index_version == 1 ? G2C_INDEX_FIXED_LEN : G2C_INDEX_FIXED_LEN_2)
+		    + msg->sec1_len + sec3->sec_len + sec4->sec_len + sec5->sec_len + G2C_INDEX_BITMAP_BYTES;
                 LOG((4, "fieldnum %d reclen %d", fieldnum, reclen));
 
                 /* Write the beginning of the index record. */
-                if ((ret = g2c_start_index_record(f, G2C_FILE_WRITE, &reclen, &bytes_to_msg, &msg->bytes_to_local,
-                                                  &bs3, &bs4, &bs5, &bs6, &bs7, &msg->bytes_in_msg, &msg->master_version,
-                                                  &msg->discipline, &fieldnum)))
-                    break;
+		if (index_version == 2)
+		{
+		    if ((ret = g2c_start_index_record_lf(f, G2C_FILE_WRITE, &reclen, &msg->bytes_to_msg, &msg->bytes_to_local,
+							 &bs3, &bs4, &bs5, &bs6, &bs7, &msg->bytes_in_msg, &msg->master_version,
+							 &msg->discipline, &fieldnum)))
+			break;
+		}
+		else
+		{
+		    int bytes_to_msg = (int)msg->bytes_to_msg;
+		    if ((ret = g2c_start_index_record(f, G2C_FILE_WRITE, &reclen, &bytes_to_msg, &msg->bytes_to_local,
+						      &bs3, &bs4, &bs5, &bs6, &bs7, &msg->bytes_in_msg, &msg->master_version,
+						      &msg->discipline, &fieldnum)))
+			break;
+		}
 
                 /* Write the section 1, identification section. */
                 if ((ret = g2c_rw_section1_metadata(f, G2C_FILE_WRITE, msg)))
@@ -598,7 +701,7 @@ read_hdr_rec1(FILE *f, int *ip, int *jp, int *kp, char *date_str, char *time_str
 	memcpy(time_str, long_time_str, G2C_INDEX_TIME_STR_LEN);
 	time_str[G2C_INDEX_TIME_STR_LEN] = 0;
     }
-    LOG((2, "str1 %s i %d j %d k %d date_str %s time_str %s", str1, i, j, k, date_str,
+    LOG((4, "str1 %s i %d j %d k %d date_str %s time_str %s", str1, i, j, k, date_str,
 	 time_str));
 
     /* Return info to caller where desired. */
@@ -632,17 +735,21 @@ read_hdr_rec1(FILE *f, int *ip, int *jp, int *kp, char *date_str, char *time_str
  * @param basename Pointer to char array of size
  * ::G2C_INDEX_BASENAME_LEN + 1 which will get the basename string from the
  * second header record. Ignored if NULL.
+ * @param index_version The version of the index, 1 for legacy, 2 to
+ * allow for > 2 GB GRIB2 files.
  *
  * @returns 0 for success, error code otherwise.
  *
  * @author Edward Hartnett @date 9/10/23
 */
 static int
-read_hdr_rec2(FILE *f, int *skipp, int *total_lenp, int *num_recp, char *basename)
+read_hdr_rec2(FILE *f, int *skipp, int *total_lenp, int *num_recp,
+	      char *basename, int *index_version)
 {
     size_t bytes_read;
     char line[G2C_INDEX_HEADER_LEN + 1];
-    int skip, total_len, num_rec;    
+    int skip;
+    int total_len, num_rec;    
     char my_basename[G2C_INDEX_BASENAME_LEN + 1];
     
     /* Read the second line of header. */
@@ -652,7 +759,8 @@ read_hdr_rec2(FILE *f, int *skipp, int *total_lenp, int *num_recp, char *basenam
     /* Scan the line. Hard! */
     {
 	char long_basename[G2C_INDEX_HEADER_LEN + 1];
-	sscanf(line, "IX1FORM: %d %d %d %s", &skip, &total_len, &num_rec, long_basename);
+	sscanf(line, "IX%dFORM: %d %d %d %s", index_version, &skip, &total_len,
+	       &num_rec, long_basename);
 	memcpy(my_basename, long_basename, G2C_INDEX_BASENAME_LEN);
 	my_basename[G2C_INDEX_BASENAME_LEN] = 0;
     }
@@ -695,6 +803,7 @@ g2c_open_index1(const char *index_file)
     unsigned char gds_val[G2C_INDEX1_GDS_VAL_LEN];
     unsigned char bms_val[G2C_INDEX1_BMS_VAL_LEN];
     unsigned char bds_val[G2C_INDEX1_BDS_VAL_LEN];
+    int index_version;
     int rec;
     int ret = G2C_NOERROR;
 
@@ -702,7 +811,7 @@ g2c_open_index1(const char *index_file)
     if (!index_file)
         return G2C_EINVAL;
 
-    LOG((1, "g2c_open_index1 index_file %s", index_file));
+    LOG((2, "g2c_open_index1 index_file %s", index_file));
 
     /* If using threading, lock the mutex. */
     MUTEX_LOCK(m);
@@ -714,12 +823,12 @@ g2c_open_index1(const char *index_file)
     /* Read header record apparently named after Steve Lord. */
     if ((ret = read_hdr_rec1(f, &i, &j, &k, date_str, time_str)))
 	return ret;
-    LOG((2, "i %d j %d k %d date_str %s time_str %s", i, j, k, date_str, time_str));
+    LOG((4, "i %d j %d k %d date_str %s time_str %s", i, j, k, date_str, time_str));
 
     /* Read second header record. */
-    if ((ret = read_hdr_rec2(f, &skip, &total_len, &num_rec, basename)))
+    if ((ret = read_hdr_rec2(f, &skip, &total_len, &num_rec, basename, &index_version)))
 	return ret;
-    LOG((2, "skip %d total_len %d num_rec %d basename %s", skip, total_len, num_rec, basename));
+    LOG((4, "skip %d total_len %d num_rec %d basename %s", skip, total_len, num_rec, basename));
 
     /* Read each index record. These is one record for each message in
        the original GRIB1 file. */
@@ -742,7 +851,7 @@ g2c_open_index1(const char *index_file)
 					   gds_val, bms_val, bds_val, NULL, NULL, NULL)))
 	    break;
 
-	LOG((3, "b2_msg %d b2_pds %d b2_gds %d b2_bms %d b2_bds %d msglen %d version %d",
+	LOG((4, "b2_msg %d b2_pds %d b2_gds %d b2_bms %d b2_bds %d msglen %d version %d",
 	     b2_msg, b2_gds, b2_pds, b2_bms, b2_bds, msglen, version));
 	printf("b2_msg %d b2_pds %d b2_gds %d b2_bms %d b2_bds %d msglen %d version %d\n",
 	       b2_msg, b2_gds, b2_pds, b2_bms, b2_bds, msglen, version);
@@ -796,7 +905,7 @@ g2c_open_index(const char *data_file, const char *index_file, int mode,
     if (strlen(data_file) > G2C_MAX_NAME)
         return G2C_ENAMETOOLONG;
 
-    LOG((1, "g2c_open_index index_file %s", index_file));
+    LOG((2, "g2c_open_index index_file %s", index_file));
 
     /* Open the index file. */
     if (!(f = fopen(index_file, "rb")))
@@ -817,6 +926,7 @@ g2c_open_index(const char *data_file, const char *index_file, int mode,
         int skip, total_len, num_rec;
         char basename[G2C_INDEX_BASENAME_LEN + 1];
         size_t file_pos = G2C_INDEX_HEADER_LEN * 2;
+	int index_version;
         int rec;
 
         /* Read the first line of header. */
@@ -836,7 +946,7 @@ g2c_open_index(const char *data_file, const char *index_file, int mode,
             memcpy(time_str, long_time_str, G2C_INDEX_TIME_STR_LEN);
             time_str[G2C_INDEX_TIME_STR_LEN] = 0;
         }
-        LOG((2, "str1 %s i %d j %d k %d date_str %s time_str %s", str1, i, j, k, date_str,
+        LOG((1, "str1 %s i %d j %d k %d date_str %s time_str %s", str1, i, j, k, date_str,
              time_str));
 
         /* Read the second line of header. */
@@ -846,17 +956,18 @@ g2c_open_index(const char *data_file, const char *index_file, int mode,
         /* Scan the line. Hard! */
         {
             char long_basename[G2C_INDEX_HEADER_LEN + 1];
-            sscanf(line, "IX1FORM: %d %d %d %s", &skip, &total_len, &num_rec, long_basename);
+	    sscanf(line, "IX%dFORM: %d %d %d %s", &index_version, &skip, &total_len,
+		   &num_rec, long_basename);
             memcpy(basename, long_basename, G2C_INDEX_BASENAME_LEN);
             basename[G2C_INDEX_BASENAME_LEN] = 0;
         }
-        LOG((2, "skip %d total_len %d num_rec %d basename %s", skip, total_len, num_rec, basename));
+        LOG((1, "skip %d total_len %d num_rec %d basename %s", skip, total_len, num_rec, basename));
 
         /* Read each index record. */
         for (rec = 0; rec < num_rec; rec++)
         {
-            int reclen, msg, local, gds, pds, drs, bms, data;
-            size_t msglen;
+            int reclen, msgint, local, gds, pds, drs, bms, data;
+            size_t msglen, msg;
             unsigned char version, discipline;
             short fieldnum;
 
@@ -868,12 +979,23 @@ g2c_open_index(const char *data_file, const char *index_file, int mode,
             }
 
             /* Read the index record. */
-            LOG((4, "reading index record at file position %ld", ftell(f)));
-            if ((ret = g2c_start_index_record(f, G2C_FILE_READ, &reclen, &msg, &local, &gds, &pds,
-                                              &drs, &bms, &data, &msglen, &version, &discipline, &fieldnum)))
-                break;
+            LOG((4, "reading index record at file position %ld, index_version %d",
+		 ftell(f), index_version));
+	    if (index_version == 1)
+	    {
+		if ((ret = g2c_start_index_record(f, G2C_FILE_READ, &reclen, &msgint, &local, &gds, &pds,
+						  &drs, &bms, &data, &msglen, &version, &discipline, &fieldnum)))
+		    break;
+		msg = msgint;
+	    }
+	    else
+	    {
+		if ((ret = g2c_start_index_record_lf(f, G2C_FILE_READ, &reclen, &msg, &local, &gds, &pds,
+						     &drs, &bms, &data, &msglen, &version, &discipline, &fieldnum)))
+		    break;
+	    }
 
-            LOG((3, "reclen %d msg %d local %d gds %d pds %d drs %d bms %d data %d "
+            LOG((1, "reclen %d msg %ld local %d gds %d pds %d drs %d bms %d data %d "
                  "msglen %ld version %d discipline %d fieldnum %d",
                  reclen, msg, local, gds, pds, drs, bms, data, msglen,
                  version, discipline, fieldnum));
